@@ -1,41 +1,25 @@
+/* SPDX-License-Identifier: BSD-2-Clause */
 /*
  * Copyright (c) 2017 - 2018, Intel Corporation
  * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- *
- * 1. Redistributions of source code must retain the above copyright notice,
- * this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright notice,
- * this list of conditions and the following disclaimer in the documentation
- * and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
- * THE POSSIBILITY OF SUCH DAMAGE.
  */
 #include <assert.h>
+#include <errno.h>
 #include <glib.h>
 #include <inttypes.h>
+#include <string.h>
 #include <unistd.h>
 
 #include <setjmp.h>
 #include <cmocka.h>
 
+#include <tss2/tss2_mu.h>
+
 #include "resource-manager.h"
-#include "tcti-echo.h"
 #include "sink-interface.h"
 #include "source-interface.h"
+#include "tcti.h"
+#include "tcti-mock.h"
 #include "tpm2-command.h"
 #include "tpm2-header.h"
 #include "util.h"
@@ -44,7 +28,6 @@ typedef struct test_data {
     AccessBroker    *access_broker;
     ResourceManager *resource_manager;
     Connection      *connection;
-    TctiEcho        *tcti_echo;
     Tpm2Command     *command;
     Tpm2Response    *response;
     gint             client_fd;
@@ -132,15 +115,16 @@ resource_manager_setup (void **state)
     GIOStream   *iostream;
     HandleMap   *handle_map;
     SessionList *session_list;
-    TSS2_RC rc;
+    Tcti *tcti = NULL;
+    TSS2_TCTI_CONTEXT *context;
+
+    context = tcti_mock_init_full ();
+    tcti = tcti_new (context, NULL);
 
     data = calloc (1, sizeof (test_data_t));
-    data->tcti_echo = tcti_echo_new (1024);
-    rc = tcti_echo_initialize (data->tcti_echo);
-    if (rc != TSS2_RC_SUCCESS)
-        g_debug ("tcti_echo_initialize FAILED");
     handle_map = handle_map_new (TPM2_HT_TRANSIENT, MAX_ENTRIES_DEFAULT);
-    data->access_broker = access_broker_new (TCTI (data->tcti_echo));
+    data->access_broker = access_broker_new (tcti);
+    g_clear_object (&tcti);
     session_list = session_list_new (SESSION_LIST_MAX_ENTRIES_DEFAULT,
                                      SESSION_LIST_MAX_ABANDONED_DEFAULT);
     data->resource_manager = resource_manager_new (data->access_broker,
@@ -205,10 +189,6 @@ resource_manager_teardown (void **state)
         g_debug ("resource_manager unref access_broker");
         g_object_unref (data->access_broker);
     }
-    if (data->tcti_echo) {
-        g_debug ("resource_manager unref TctiEcho");
-        g_object_unref (data->tcti_echo);
-    }
     if (data->connection) {
         g_debug ("resource_manager unref Connection");
         g_object_unref (data->connection);
@@ -398,6 +378,123 @@ resource_manager_load_handles_test(void **state)
         assert_int_equal (phandles [i], handle_ret);
     }
 }
+/*
+ * This setup function calls the 'resource_manager_setup' function to create
+ * the ResourceManager object etc. It then creates a Tpm2Response object
+ * approprite for use in testing the 'get_cap_post_process' function.
+ */
+int
+resource_manager_setup_getcap (void **state)
+{
+    int setup_ret;
+    test_data_t *data = NULL;
+    size_t offset = 0, buf_size = 0;
+    uint8_t *buf = NULL;
+    TSS2_RC rc;
+    /* structures that define the Tpm2Response object used in GetCap tests */
+    TPMI_YES_NO yes_no = TPM2_NO;
+    TPMS_CAPABILITY_DATA cap_data = {
+        .capability = TPM2_CAP_TPM_PROPERTIES,
+        /* TPMU_CAPABILITIES, union type defined by 'capability' selector */
+        .data = {
+            /* TPML_TAGGED_TPM_PROPERTY */
+            .tpmProperties = {
+                .count = 1,
+                /* TPMS_TAGGED_PROPERTY */
+                .tpmProperty = {
+                    {
+                      .property = TPM2_PT_CONTEXT_GAP_MAX,
+                      .value = 0xff,
+                    }
+                }
+            }
+        }
+    };
+
+    setup_ret = resource_manager_setup (state);
+    if (setup_ret != 0) {
+        g_critical ("%s: resource_manager_setup failed", __func__);
+        return setup_ret;
+    }
+    data = (test_data_t*)*state;
+    buf_size = TPM2_MAX_RESPONSE_SIZE;
+    g_debug ("%s: sizeof buffer required for TPMS_CAPABILITY_DATA: 0x%zx",
+             __func__, offset);
+
+    buf = calloc (1, buf_size);
+    if (buf == NULL) {
+        g_critical ("%s: failed to allocate buffer: %s", __func__,
+                    strerror (errno));
+        return 1;
+    }
+    offset = TPM_HEADER_SIZE;
+    rc = Tss2_MU_BYTE_Marshal (yes_no, buf, buf_size, &offset);
+    if (rc != TSS2_RC_SUCCESS) {
+        g_critical ("%s: failed to marshal TPMI_YES_NO into response body, "
+                    "rc: 0x%" PRIx32, __func__, rc);
+        return 1;
+    }
+    rc = Tss2_MU_TPMS_CAPABILITY_DATA_Marshal (&cap_data, buf, buf_size, &offset);
+    if (rc != TSS2_RC_SUCCESS) {
+        g_critical ("%s: failed to marshal TPMS_CAPABILITY_DATA into response "
+                    "body, rc: 0x%" PRIx32, __func__, rc);
+        return 1;
+    }
+    rc = tpm2_header_init (buf,
+                           buf_size,
+                           TPM2_ST_NO_SESSIONS,
+                           offset,
+                           TSS2_RC_SUCCESS);
+    if (rc != TSS2_RC_SUCCESS) {
+        g_critical ("%s: failed to initialize TPM2 Header, rc: 0x%" PRIx32,
+                    __func__, rc);
+        return 1;
+    }
+    data->response = tpm2_response_new (data->connection, buf, offset, TPM2_CC_GetCapability);
+    return setup_ret;
+}
+/*
+ * Test 'getcap_post_process' function to ensure that TPM2_PT_CONTEXT_GAP_MAX
+ * property is properly modified from its initial value of UINT8_MAX to
+ * UINT32_MAX.
+ */
+void
+resource_manager_getcap_gap_max_test (void **state)
+{
+    test_data_t *data = (test_data_t*)*state;
+    size_t offset = TPM_HEADER_SIZE, buf_size;
+    uint8_t *buf;
+    TSS2_RC rc;
+    TPMI_YES_NO yes_no = TPM2_NO;
+    TPMS_CAPABILITY_DATA cap_data = { 0 };
+
+    /* get buffer & size from Tpm2Response created in setup function */
+    buf = tpm2_response_get_buffer (data->response);
+    buf_size = tpm2_response_get_size (data->response);
+    assert_non_null (buf);
+    /* verify defaults from setup function */
+    rc = Tss2_MU_BYTE_Unmarshal (buf, buf_size, &offset, &yes_no);
+    assert_int_equal (rc, TSS2_RC_SUCCESS);
+    rc = Tss2_MU_TPMS_CAPABILITY_DATA_Unmarshal (buf,
+                                                 buf_size,
+                                                 &offset,
+                                                 &cap_data);
+    assert_int_equal (rc, TSS2_RC_SUCCESS);
+    assert_int_equal (cap_data.data.tpmProperties.tpmProperty [0].value, UINT8_MAX);
+    offset = TPM_HEADER_SIZE;
+    /* execute function under test */
+    rc = get_cap_post_process (data->response);
+    assert_int_equal (rc, TSS2_RC_SUCCESS);
+    rc = Tss2_MU_BYTE_Unmarshal (buf, buf_size, &offset, &yes_no);
+    assert_int_equal (rc, TSS2_RC_SUCCESS);
+    rc = Tss2_MU_TPMS_CAPABILITY_DATA_Unmarshal (buf,
+                                                 buf_size,
+                                                 &offset,
+                                                 &cap_data);
+    assert_int_equal (rc, TSS2_RC_SUCCESS);
+    /* verify property was modified by the RM */
+    assert_int_equal (cap_data.data.tpmProperties.tpmProperty [0].value, UINT32_MAX);
+}
 int
 main (void)
 {
@@ -423,6 +520,9 @@ main (void)
         cmocka_unit_test_setup_teardown (resource_manager_load_handles_test,
                                          resource_manager_setup_two_transient_handles,
                                          resource_manager_teardown),
+        cmocka_unit_test_setup_teardown (resource_manager_getcap_gap_max_test,
+                                         resource_manager_setup_getcap,
+                                         resource_manager_teardown),
     };
     return cmocka_run_group_tests (tests, NULL, NULL);
 }
